/* Hantering av varukorgen och önskelistan i uppgift U7.
   Detta är förenklade listor, eftersom uppgiften inte handlar om listornas struktur och innehåll. Istället är det interaktionen för att hantera listorna (flytta varor mellan listorna och slänga varor i soptunnan), som uppgiften går ut på, vilket är kod som du skriver i filen list.js. */
// --------------------------------------------------
// Definition av en class för att skapa listor (varukorg och önskelista).
class ListObject {
    constructor (elem, storageName) {
        // Tilldelning av egenskaper
        this.listElem = elem; // Element där listan ska visas (null, om listan ej ska visas)
        this.storageName = storageName; // Namn då listan sparas i localStorage
        this.list = [];       // Ny tom array för listan
        // Kod som utförs, för att initiera objektet
        this.getList();       // Läs in listan från localStorage, ifall den finns sparad där
        this.showList();      // Skriv ut listan i listElem på webbsidan
    } // Slut constructor
    // ----------------------
    // Om produkten med artikelnummer anr inte redan finns i listan, läggs den in som en ny vara.
    addItem(anr) {
        for (let i = 0; i < this.list.length; i++) {
            if (this.list[i].artnr == anr) return; // Produkten finns redan i listan
        }
        this.list.push( // Lägg till ett objekt för den nya produkten sist i listan
            {
                artnr: anr,
                quantity: 1
            }
        );
        this.showList(); // Visa nya listan
        this.saveList(); // Spara listan i localStorage
    } // Slut addItem
    // ----------------------
    // Ta bort varan med artikelnummer anr.
    removeItem(anr) {
        let ix = -1;
        for (let i = 0; i < this.list.length; i++) { // Sök efter artikelnumret i listan
            if (this.list[i].artnr == anr) {
                ix = i;
                break;
            }
        }
        if (ix > -1) { // Varan finns i listan, ...
            this.list.splice(ix, 1); // ... så ta bort den
            this.showList(); // Visa nya listan
            this.saveList(); // Spara listan i localStorage
        }
    } // Slut removeItem
    // ----------------------
    // Skapa HTML-kod för visning av listan.
    showList() {
        if (this.listElem == null) return; // Det finns inget element att visa listan i
        this.listElem.innerHTML = ""; // Ta bort visning av gamla listan
        for (let i = 0; i < this.list.length; i++) { // Gå igenom alla varor i listan
            let prod = this.getProduct(this.list[i].artnr); // Objekt för aktuell produkt
            let divElem = document.createElement("div");   // Element för produkten, som innehåller ...
            let h4Elem = document.createElement("h4");     // ... h4-element för produktnamn ...
            let pElem = document.createElement("p");       // ... p-element för pris ...
            let spanElem = document.createElement("span"); // ... och dolt span-element med artikelnummer
            h4Elem.innerText = prod.name;
            pElem.innerText = prod.price + " kr";
            spanElem.innerText = prod.artnr; // Artikelnumret behövs då varan flyttas eller slängs
            spanElem.className = "artnr";    // Elementet döljs med CSS-koden
            divElem.appendChild(h4Elem);
            divElem.appendChild(pElem);
            divElem.appendChild(spanElem);
            this.listElem.appendChild(divElem);
            divElem.addEventListener("pointerdown", dragStart); // dragStart ska läggas till i list.js
        }
    } // Slut showList
    // ----------------------
    // Returnera objekt för produkten med artikelnumret artnr. Hämtas ur registret i data.js. Finns det ingen sådan produkt, returneras null.
    getProduct(anr) {
        let products = []; // Array med lista av produkter i aktuell kategori
        switch (anr[0]) { // Välj produktvariabel (som finns i data.js) utifrån artikelnumrets första tecken.
            case "c": products = chocolate.products; break;
            case "k": products = caramel.products; break;
            case "m": products = softcandy.products; break;
        }
        for (let i = 0; i < products.length; i++) { // Sök produkten med artikelnumret som finns i anr
            if (products[i].artnr == anr) return products[i]; // Produktens objekt
        }
        return null; // Produkten finns inte
    } // Slut getProduct
    // ----------------------
    // Konvertera listan till en sträng m.h.a. JSON och spara i localStorage.
    saveList() {
        let data = JSON.stringify(this.list);
        localStorage.setItem(this.storageName, data);
    } // Slut saveList
    // ----------------------
    // Läs in från localStorage (om listan finns där) och konvertera till listans datatyp m.h.a. JSON.
    getList() {
        let data = localStorage.getItem(this.storageName);
        if (data) {
            this.list = JSON.parse(data);
        }
    } // Slut getList
    // ----------------------
} // Slut class ListObject
// --------------------------------------------------
